@@ -4,6 +4,8 @@
 # version of Python, then hand over to the actual Hypothesis build runner (which
 # is written in Python instead of bash).
 
+if [ -n "${CI:-}" ] ; then echo "::group::Build setup" ; fi
+
 set -o xtrace
 set -o errexit
 set -o nounset
@@ -23,8 +25,8 @@ if [ -n "${GITHUB_ACTIONS-}" ] || [ -n "${CODESPACES-}" ] ; then
 else
     # Otherwise, we install it from scratch
     # NOTE: tooling keeps this version in sync with ci_version in tooling
-    "$SCRIPTS/ensure-python.sh" 3.10.13
-    PYTHON=$(pythonloc 3.10.13)/bin/python
+    "$SCRIPTS/ensure-python.sh" 3.10.14
+    PYTHON=$(pythonloc 3.10.14)/bin/python
 fi
 
 TOOL_REQUIREMENTS="$ROOT/requirements/tools.txt"
@@ -44,4 +46,6 @@ if ! "$TOOL_PYTHON" -m hypothesistooling check-installed ; then
   "$TOOL_PYTHON" -m pip install --no-warn-script-location -r requirements/tools.txt
 fi
 
+if [ -n "${CI:-}" ] ; then echo "::endgroup::" ; fi
+
 "$TOOL_PYTHON" -m hypothesistooling "$@"
