@@ -17,6 +17,7 @@ import pytest
 
 from hypothesis import (
     HealthCheck,
+    Phase,
     assume,
     given,
     note,
@@ -26,8 +27,9 @@ from hypothesis import (
 )
 from hypothesis.errors import InvalidArgument, UnsatisfiedAssumption
 from hypothesis.extra import numpy as nps
+from hypothesis.strategies._internal.lazy import unwrap_strategies
 
-from tests.common.debug import find_any, minimal
+from tests.common.debug import check_can_generate_examples, find_any, minimal
 from tests.common.utils import fails_with, flaky
 
 ANY_SHAPE = nps.array_shapes(min_dims=0, max_dims=32, min_side=0, max_side=32)
@@ -60,11 +62,7 @@ def test_generates_and_minimizes():
 
 
 def test_can_minimize_large_arrays():
-    x = minimal(
-        nps.arrays("uint32", 100),
-        lambda x: np.any(x) and not np.all(x),
-        timeout_after=60,
-    )
+    x = minimal(nps.arrays("uint32", 100), lambda x: np.any(x) and not np.all(x))
     assert np.logical_or(x == 0, x == 1).all()
     assert np.count_nonzero(x) in (1, len(x) - 1)
 
@@ -142,7 +140,7 @@ def test_minimise_array_shapes(min_dims, dim_range, min_side, side_range):
     "kwargs", [{"min_side": 100}, {"min_dims": 15}, {"min_dims": 32}]
 )
 def test_interesting_array_shapes_argument(kwargs):
-    nps.array_shapes(**kwargs).example()
+    check_can_generate_examples(nps.array_shapes(**kwargs))
 
 
 @given(nps.scalar_dtypes())
@@ -257,7 +255,7 @@ def test_array_values_are_unique(arr):
 def test_cannot_generate_unique_array_of_too_many_elements():
     strat = nps.arrays(dtype=int, elements=st.integers(0, 5), shape=10, unique=True)
     with pytest.raises(InvalidArgument):
-        strat.example()
+        check_can_generate_examples(strat)
 
 
 @given(
@@ -347,7 +345,11 @@ np_version = tuple(int(x) for x in np.__version__.split(".")[:2])
 
 @pytest.mark.parametrize("fill", [False, True])
 # Overflowing elements deprecated upstream in Numpy 1.24 :-)
-@fails_with(InvalidArgument if np_version < (1, 24) else DeprecationWarning)
+@fails_with(
+    InvalidArgument
+    if np_version < (1, 24)
+    else (DeprecationWarning if np_version < (2, 0) else OverflowError)
+)
 @given(st.data())
 def test_overflowing_integers_are_deprecated(fill, data):
     kw = {"elements": st.just(300)}
@@ -909,7 +911,7 @@ def test_mutually_broadcastable_shapes_only_singleton_is_valid(
         assert all(i == 1 for i in shape[-len(base_shape) :])
 
 
-@settings(deadline=None)
+@settings(deadline=None, suppress_health_check=[HealthCheck.too_slow])
 @given(
     shape=nps.array_shapes(min_dims=0, max_dims=3, min_side=0, max_side=5),
     max_dims=st.integers(0, 6),
@@ -963,7 +965,7 @@ def test_mutually_broadcastable_shapes_can_generate_arbitrary_ndims(
     )
 
 
-@settings(deadline=None)
+@settings(deadline=None, suppress_health_check=list(HealthCheck))
 @given(
     base_shape=nps.array_shapes(min_dims=0, max_dims=3, min_side=0, max_side=2),
     max_dims=st.integers(1, 4),
@@ -1097,20 +1099,22 @@ def test_advanced_integer_index_can_generate_any_pattern(shape, data):
         target(float(np.sum(target_array == selected)), label="elements correct")
         return np.all(target_array == selected)
 
-    find_any(
+    minimal(
         nps.integer_array_indices(shape, result_shape=st.just(target_array.shape)),
         index_selects_values_in_order,
-        settings(max_examples=10**6),
+        settings(max_examples=10**6, phases=[Phase.generate, Phase.target]),
     )
 
 
 @pytest.mark.parametrize(
     "condition",
     [
-        lambda ix: Ellipsis in ix,
-        lambda ix: Ellipsis not in ix,
-        lambda ix: np.newaxis in ix,
-        lambda ix: np.newaxis not in ix,
+        lambda ix: isinstance(ix, tuple) and Ellipsis in ix,
+        lambda ix: isinstance(ix, tuple) and Ellipsis not in ix,
+        lambda ix: isinstance(ix, tuple) and np.newaxis in ix,
+        lambda ix: isinstance(ix, tuple) and np.newaxis not in ix,
+        lambda ix: ix is Ellipsis,
+        lambda ix: ix == np.newaxis,
     ],
 )
 def test_basic_indices_options(condition):
@@ -1238,3 +1242,19 @@ def test_no_recursion_in_multi_line_reprs_issue_3560(data):
             dtype=float,
         ).map(lambda x: x)
     )
+
+
+def test_infers_elements_and_fill():
+    # Regression test for https://github.com/HypothesisWorks/hypothesis/issues/3900
+    # We only infer a fill strategy if the elements_strategy has reusable values,
+    # and the interaction of two performance fixes broke this.  Oops...
+    s = unwrap_strategies(nps.arrays(dtype=np.uint32, shape=1))
+    assert isinstance(s, nps.ArrayStrategy)
+    assert repr(s.element_strategy) == f"integers(0, {2**32-1})"
+    assert repr(s.fill) == f"integers(0, {2**32-1})"
+
+    # But we _don't_ infer a fill if the elements strategy is non-reusable
+    elems = st.builds(lambda x: x * 2, st.integers(1, 10)).map(np.uint32)
+    assert not elems.has_reusable_values
+    s = unwrap_strategies(nps.arrays(dtype=np.uint32, shape=1, elements=elems))
+    assert s.fill.is_empty
